package dev.shadowsoffire.apotheosis.adventure.affix.effect;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import com.mojang.datafixers.kinds.App;
import com.mojang.datafixers.kinds.Applicative;
import com.mojang.datafixers.util.Function6;
import com.mojang.serialization.Codec;
import com.mojang.serialization.codecs.RecordCodecBuilder;
import dev.shadowsoffire.apotheosis.adventure.affix.Affix;
import dev.shadowsoffire.apotheosis.adventure.affix.AffixType;
import dev.shadowsoffire.apotheosis.adventure.loot.LootCategory;
import dev.shadowsoffire.apotheosis.adventure.loot.LootRarity;
import dev.shadowsoffire.placebo.codec.PlaceboCodecs;
import java.util.Map;
import java.util.Set;
import net.minecraft.core.BlockPos;
import net.minecraft.network.chat.Component;
import net.minecraft.network.chat.MutableComponent;
import net.minecraft.util.StringUtil;
import net.minecraft.world.damagesource.DamageSource;
import net.minecraft.world.effect.MobEffect;
import net.minecraft.world.effect.MobEffectInstance;
import net.minecraft.world.effect.MobEffectUtil;
import net.minecraft.world.entity.Entity;
import net.minecraft.world.entity.LivingEntity;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.entity.projectile.AbstractArrow;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.level.LevelAccessor;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.world.phys.EntityHitResult;
import net.minecraft.world.phys.HitResult;
import net.minecraftforge.registries.ForgeRegistries;

public class PotionAffix extends Affix {
  public static final Codec<dev.shadowsoffire.apotheosis.adventure.affix.effect.PotionAffix> CODEC;
  
  protected final MobEffect effect;
  
  protected final Target target;
  
  protected final Map<LootRarity, EffectData> values;
  
  @Deprecated(forRemoval = true, since = "6.3.0")
  protected final int cooldown;
  
  protected final Set<LootCategory> types;
  
  protected final boolean stackOnReapply;
  
  static {
    CODEC = RecordCodecBuilder.create(inst -> inst.group((App)ForgeRegistries.MOB_EFFECTS.getCodec().fieldOf("mob_effect").forGetter(()), (App)Target.CODEC.fieldOf("target").forGetter(()), (App)LootRarity.mapCodec(EffectData.CODEC).fieldOf("values").forGetter(()), (App)PlaceboCodecs.nullableField((Codec)Codec.INT, "cooldown", Integer.valueOf(0)).forGetter(()), (App)LootCategory.SET_CODEC.fieldOf("types").forGetter(()), (App)PlaceboCodecs.nullableField((Codec)Codec.BOOL, "stack_on_reapply", Boolean.valueOf(false)).forGetter(())).apply((Applicative)inst, dev.shadowsoffire.apotheosis.adventure.affix.effect.PotionAffix::new));
  }
  
  public PotionAffix(MobEffect effect, Target target, Map<LootRarity, EffectData> values, int cooldown, Set<LootCategory> types, boolean stackOnReapply) {
    super(AffixType.ABILITY);
    this.effect = effect;
    this.target = target;
    this.values = values;
    this.cooldown = cooldown;
    this.types = types;
    this.stackOnReapply = stackOnReapply;
  }
  
  public MutableComponent getDescription(ItemStack stack, LootRarity rarity, float level) {
    MobEffectInstance inst = ((EffectData)this.values.get(rarity)).build(this.effect, level);
    MutableComponent comp = this.target.toComponent(new Object[] { toComponent(inst) });
    int cooldown = getCooldown(rarity);
    if (cooldown != 0) {
      MutableComponent mutableComponent = Component.m_237110_("affix.apotheosis.cooldown", new Object[] { StringUtil.m_14404_(cooldown) });
      comp = comp.m_130946_(" ").m_7220_((Component)mutableComponent);
    } 
    if (this.stackOnReapply)
      comp = comp.m_130946_(" ").m_7220_((Component)Component.m_237115_("affix.apotheosis.stacking")); 
    return comp;
  }
  
  public Component getAugmentingText(ItemStack stack, LootRarity rarity, float level) {
    MobEffectInstance inst = ((EffectData)this.values.get(rarity)).build(this.effect, level);
    MutableComponent comp = this.target.toComponent(new Object[] { toComponent(inst) });
    MobEffectInstance min = ((EffectData)this.values.get(rarity)).build(this.effect, 0.0F);
    MobEffectInstance max = ((EffectData)this.values.get(rarity)).build(this.effect, 1.0F);
    if (min.m_19564_() != max.m_19564_()) {
      MutableComponent mutableComponent1 = (min.m_19564_() == 0) ? Component.m_237113_("I") : Component.m_237115_("potion.potency." + min.m_19564_());
      MutableComponent mutableComponent2 = Component.m_237115_("potion.potency." + max.m_19564_());
      comp.m_7220_((Component)valueBounds((Component)mutableComponent1, (Component)mutableComponent2));
    } 
    if (!this.effect.m_8093_() && min.m_19557_() != max.m_19557_()) {
      Component minComp = MobEffectUtil.m_267641_(min, 1.0F);
      Component maxComp = MobEffectUtil.m_267641_(max, 1.0F);
      comp.m_7220_((Component)valueBounds(minComp, maxComp));
    } 
    int cooldown = getCooldown(rarity);
    if (cooldown != 0) {
      MutableComponent mutableComponent = Component.m_237110_("affix.apotheosis.cooldown", new Object[] { StringUtil.m_14404_(cooldown) });
      comp = comp.m_130946_(" ").m_7220_((Component)mutableComponent);
    } 
    if (this.stackOnReapply)
      comp = comp.m_130946_(" ").m_7220_((Component)Component.m_237115_("affix.apotheosis.stacking")); 
    return (Component)comp;
  }
  
  public boolean canApplyTo(ItemStack stack, LootCategory cat, LootRarity rarity) {
    return ((this.types.isEmpty() || this.types.contains(cat)) && this.values.containsKey(rarity));
  }
  
  public void doPostHurt(ItemStack stack, LootRarity rarity, float level, LivingEntity user, Entity attacker) {
    if (this.target == Target.HURT_SELF) {
      applyEffect(user, rarity, level);
    } else if (this.target == Target.HURT_ATTACKER && 
      attacker instanceof LivingEntity) {
      LivingEntity tLiving = (LivingEntity)attacker;
      applyEffect(tLiving, rarity, level);
    } 
  }
  
  public void doPostAttack(ItemStack stack, LootRarity rarity, float level, LivingEntity user, Entity target) {
    if (this.target == Target.ATTACK_SELF) {
      applyEffect(user, rarity, level);
    } else if (this.target == Target.ATTACK_TARGET && 
      target instanceof LivingEntity) {
      LivingEntity tLiving = (LivingEntity)target;
      applyEffect(tLiving, rarity, level);
    } 
  }
  
  public void onBlockBreak(ItemStack stack, LootRarity rarity, float level, Player player, LevelAccessor world, BlockPos pos, BlockState state) {
    if (this.target == Target.BREAK_SELF)
      applyEffect((LivingEntity)player, rarity, level); 
  }
  
  public void onArrowImpact(AbstractArrow arrow, LootRarity rarity, float level, HitResult res, HitResult.Type type) {
    if (this.target == Target.ARROW_SELF) {
      Entity entity = arrow.m_19749_();
      if (entity instanceof LivingEntity) {
        LivingEntity owner = (LivingEntity)entity;
        applyEffect(owner, rarity, level);
      } 
    } else if (this.target == Target.ARROW_TARGET && 
      type == HitResult.Type.ENTITY) {
      Entity entity = ((EntityHitResult)res).m_82443_();
      if (entity instanceof LivingEntity) {
        LivingEntity target = (LivingEntity)entity;
        applyEffect(target, rarity, level);
      } 
    } 
  }
  
  public float onShieldBlock(ItemStack stack, LootRarity rarity, float level, LivingEntity entity, DamageSource source, float amount) {
    if (this.target == Target.BLOCK_SELF) {
      applyEffect(entity, rarity, level);
    } else if (this.target == Target.BLOCK_ATTACKER) {
      Entity entity1 = source.m_7640_();
      if (entity1 instanceof LivingEntity) {
        LivingEntity target = (LivingEntity)entity1;
        applyEffect(target, rarity, level);
      } 
    } 
    return amount;
  }
  
  protected int getCooldown(LootRarity rarity) {
    EffectData data = this.values.get(rarity);
    if (data.cooldown != -1)
      return data.cooldown; 
    return this.cooldown;
  }
  
  private void applyEffect(LivingEntity target, LootRarity rarity, float level) {
    if (target.m_9236_().m_5776_())
      return; 
    int cooldown = getCooldown(rarity);
    if (cooldown != 0 && isOnCooldown(getId(), cooldown, target))
      return; 
    EffectData data = this.values.get(rarity);
    MobEffectInstance inst = target.m_21124_(this.effect);
    if (this.stackOnReapply && inst != null) {
      if (inst != null) {
        MobEffectInstance newInst = new MobEffectInstance(this.effect, (int)Math.max(inst.m_19557_(), data.duration.get(level)), (int)((inst.m_19564_() + 1) + data.amplifier.get(level)));
        target.m_7292_(newInst);
      } 
    } else {
      target.m_7292_(data.build(this.effect, level));
    } 
    startCooldown(getId(), target);
  }
  
  public Codec<? extends Affix> getCodec() {
    return (Codec)CODEC;
  }
  
  public static Component toComponent(MobEffectInstance inst) {
    MutableComponent mutablecomponent = Component.m_237115_(inst.m_19576_());
    MobEffect mobeffect = inst.m_19544_();
    if (inst.m_19564_() > 0)
      mutablecomponent = Component.m_237110_("potion.withAmplifier", new Object[] { mutablecomponent, Component.m_237115_("potion.potency." + inst.m_19564_()) }); 
    if (inst.m_19557_() > 20)
      mutablecomponent = Component.m_237110_("potion.withDuration", new Object[] { mutablecomponent, MobEffectUtil.m_267641_(inst, 1.0F) }); 
    return (Component)mutablecomponent.m_130940_(mobeffect.m_19483_().m_19497_());
  }
}
